# using Kivy to create a GUI for the publisher and subscriber
import os
import time
from threading import Thread
import pika

from kivy.app import App
from kivy.core.window import Window
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.label import Label

from dotenv import load_dotenv
import docker

load_dotenv()  # take environment variables from .env.


class AIJMessagingServer(App):
    """
    A class to create a GUI for the publisher and subscriber
    """

    def build(self):
        """
        Build the GUI for starting RabbitMQ with only one button and a label
        """
        self.title = 'AIJ News'
        self.icon = 'logo.png'

        # create a layout for the publisher
        self.publisher_layout = BoxLayout(orientation='vertical', padding=10)

        # create a label for the publisher
        self.publisher_label = Label(
            text='RabbitMQ Messaging Server', size_hint=(1, 0.90))

        # create a button for the publisher
        self.publisher_button = Button(text='Start', size_hint=(1, 0.10))

        # change the background color of the button to red
        self.publisher_button.background_color = (1, 0, 0, 1)

        # add the label, text input and button to the publisher layout
        self.publisher_layout.add_widget(self.publisher_label)
        self.publisher_layout.add_widget(self.publisher_button)

        # bind the button click event to the start method
        self.publisher_button.bind(on_press=self.start)

        return self.publisher_layout

    def set_button_bg(self, rgb, *args):
        """
        The method to change the background color of the button
        """
        # change the background color of the button to red
        self.publisher_button.background_color = rgb

    def start(self, instance):
        """
        Initialize the RabbitMQ server
        """
        client = docker.from_env()
        # docker run -d --hostname my-rabbit --name some-rabbit rabbitmq:3
        client.containers.run(
            "rabbitmq:3", 
            name='aij-messaging-server',
            hostname='aij-messaging-server',
            ports={'5672/tcp': 5672},
            environment={'RABBITMQ_DEFAULT_USER': 'admin', 'RABBITMQ_DEFAULT_PASS': 'admin'},
            volumes={
                
            },
            detach=True
        )

        # print all containers
        containers = client.containers.list()

        for container in containers:
            print(
                container.name,
                container.id,
                container.status
            )

        # change the text of the button to 'Stop'
        self.publisher_button.text = 'Stop'

        # bind the button click event to the stop method
        self.publisher_button.bind(on_press=self.stop)

        # change the background color of the button to green
        self.publisher_button.background_color = (0, 1, 0, 1)

        # bind the keyboard key event to the on_keyboard method
        Window.bind(on_keyboard=self.on_keyboard)

    def stop(self, instance):
        """
        Stop the RabbitMQ server
        """
        # stop the container with the name 'aij-messaging-server' and remove it
        client = docker.from_env()
        container = client.containers.get('aij-messaging-server')
        container.stop()

        # change the text of the button to 'Start'
        self.publisher_button.text = 'Start'

        # bind the button click event to the start method
        self.publisher_button.bind(on_press=self.start)

        # change the background color of the button to red
        self.publisher_button.background_color = (1, 0, 0, 1)

    def listen(self):
        # connect to RabbitMQ server and read logs using pika
        connection = pika.BlockingConnection(
            pika.ConnectionParameters('localhost'))
        channel = connection.channel()
        # print all the channel logs
        channel.basic_consume(
            queue='logs', on_message_callback=self.print_logs, auto_ack=True)

        # start listening to the logs
        channel.start_consuming()

    def print_logs(self, ch, method, properties, body):
        """
        The method to print the logs
        """
        # print the logs
        response = body.decode()
        self.publisher_label.text = response

    # add a keyboard key event to stop the RabbitMQ server when the app is closed
    def on_keyboard(self, window, key, scancode, codepoint, modifier):
        """
        The method to stop the RabbitMQ server when the app is closed
        The key event is triggered when the escape key is pressed
        """
        if key == 27:
            os.system('rabbitmqctl stop')
            return True


def main():
    # start an instance of AIJMessagingServer class
    msg = AIJMessagingServer()
    msg.run()


if __name__ == '__main__':
    main()



import json
import logging as log
import time
from threading import Timer, Thread
import logging as log
import os
import time
import pika

from newsapi import NewsApiClient
from newsapi.newsapi_exception import NewsAPIException


# using Kivy to create a GUI for the publisher and subscriber
from kivy.app import App
from kivy.core.window import Window
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.label import Label


from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

class NewsPublisher:
    """
    A class to publish news articles to a RabbitMQ queue using the NewsAPI
    @param api_key: the api key to access the NewsAPI
    @param host: the host name of the RabbitMQ server
    @param queue_name: the name of the queue to publish the news articles
    """

    def __init__(self, api_key, host='localhost', queue_name='news_stream'):
        self.api = NewsApiClient(api_key=api_key)
        self.host = host
        self.queue_name = queue_name
        # set up a connection to RabbitMQ
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host=self.host))
        self.channel = self.connection.channel()
        self.sources = 'bbc-news, cnn, fox-news, google-news, time, wired, the-new-york-times, ' \
            'the-wall-street-journal, the-washington-post, usa-today, abc-news, associated-press, ' \
            'bloomberg, business-insider, cbs-news, cnbc, entertainment-weekly, espn, fortune, fox-sports, ' \
            'mtv-news, national-geographic, nbc-news, new-scientist, newsweek, politico, reddit-r-all, ' \
            'reuters, the-hill, the-huffington-post, the-verge, the-washington-times, vice-news'

        try:
            self.articles = self.api.get_everything(sources=self.sources)
            self.headlines = self.api.get_top_headlines(sources=self.sources)
        except NewsAPIException as api_exception:
            log.error(
                f"Could not request results from NewsAPI; {api_exception}")

    def publish(self):
        """
        Publish the news articles to the RabbitMQ queue one by one and save the news to the database
        """
        try:
            for _article in self.articles['articles']:
                _body = json.dumps(_article).encode('utf-8')
                self.channel.basic_publish(
                    exchange='', routing_key=self.queue_name, body=_body)
                log.info(
                    f"Published a news article to the queue: {_article['title']}")
                # wait for 1 second before publishing the next article
                time.sleep(1)

            for _headline in self.headlines['articles']:
                _body = json.dumps(_headline).encode('utf-8')
                self.channel.basic_publish(
                    exchange='', routing_key=f"{self.queue_name}_headlines", body=_body)
                log.info(
                    f"Published a news headline to the queue: {_headline['title']}")
                # wait for 1 second before publishing the next article
                time.sleep(1)
        except NewsAPIException as api_exception:
            log.error(
                f"Could not request results from NewsAPI; {api_exception}")

        # do not close the connection until the message is delivered
        if self.connection.is_open:
            self.connection.close()

        # call the function again after 100 seconds because there are max. 100 results per page
        Timer(60 * 5, self.publish).start()

    def destroy(self):
        """
        Destroy the connection to the RabbitMQ server
        """
        self.connection.close()


class NewsSubscriber:
    """
    This class implements a RabbitMQ consumer.
    """

    def __init__(self, host='localhost', queue_name='news_stream', callback=None):
        """
        This method initializes the consumer.
        """
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host))
        self.channel = self.connection.channel()
        self.channel.queue_declare(queue=queue_name)
        self.callback = callback
        self.queue_name = queue_name

    def subscribe(self):
        """
        This method starts consuming messages from the RabbitMQ queue.
        """
        self.channel.basic_consume(
            queue=self.queue_name, on_message_callback=self.callback, auto_ack=True)
        self.channel.start_consuming()

    def destroy(self):
        """
        This method closes the connection to the RabbitMQ server.
        """
        if self.connection.is_open:
            self.connection.close()


class AIJKivy(App):
    """
    A class to create a GUI for the publisher and subscriber
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # create an API key for the News API
        api_key = os.environ.get('NEWSAPI_ORG')
        if not api_key:
            raise ValueError('API key is not set')

        # create an instance of the NewsPublisher class
        self.publisher = NewsPublisher(
            api_key=api_key,
            host='localhost',
            queue_name='aij_news'
        )

        # create an instance of the NewsSubscriber class
        self.subscriber = NewsSubscriber(
            host='localhost',
            queue_name='aij_news',
            callback=self.print_news
        )

        self.title = 'AIJ News'
        self.icon = 'logo.png'

        # create a layout for the publisher
        self.publisher_layout = BoxLayout(orientation='vertical', padding=10)
        self.publisher_label = Label(text='Publisher', size_hint=(1, 0.90))
        self.publisher_button = Button(text='Publish', size_hint=(1, 0.10))

        # create a layout for the subscriber
        self.subscriber_layout = BoxLayout(orientation='vertical', padding=10)
        self.subscriber_label = Label(text='Subscriber', size_hint=(1, 0.90))
        self.subscriber_button = Button(text='Subscribe', size_hint=(1, 0.10))

        # create a layout for the main window
        self.main_layout = BoxLayout(orientation='horizontal', padding=10)

    def build(self):
        """
        Build the GUI for the publisher and subscriber
        """

        # add the label, text input and button to the publisher layout
        self.publisher_layout.add_widget(self.publisher_label)
        self.publisher_layout.add_widget(self.publisher_button)
        # bind the button to the publish method
        self.publisher_button.bind(on_press=self.publish)

        # add the label, text input and button to the subscriber layout
        self.subscriber_layout.add_widget(self.subscriber_label)
        self.subscriber_layout.add_widget(self.subscriber_button)
        # bind the button to the subscribe method
        self.subscriber_button.bind(on_press=self.subscribe)

        # add the publisher and subscriber layouts to the main layout
        self.main_layout.add_widget(self.publisher_layout)
        self.main_layout.add_widget(self.subscriber_layout)

        Window.bind(on_keyboard=self.on_keyboard)

        return self.main_layout

    def publish(self, *args):
        """
        The method to publish the news articles to the RabbitMQ queue
        """

        # starts a thread to publish the news articles to the RabbitMQ queue
        news_publisher_thread = Thread(target=self.publisher.publish)

        # start the thread
        news_publisher_thread.start()

    def subscribe(self, *args):
        """
        The method to subscribe to the RabbitMQ queue and receive the news articles
        """

        # starts a thread to subscribe to the RabbitMQ queue and receive the news articles
        news_subscriber_thread = Thread(target=self.subscriber.subscribe)

        # start the thread
        news_subscriber_thread.start()

    def print_news(self, ch, method, properties, body):
        """
        This method is called when a message is received from the RabbitMQ queue.
        """
        response = body.decode('utf-8')
        print(
            f'-------------------------\n'
            f'{response}'
            f'-------------------------\n'
        )
        time.sleep(1)

    # add a on destroy method to stop the RabbitMQ server when the app is closed
    def on_stop(self):
        """
        The method to stop the RabbitMQ server when the app is closed
        """
        os.system('rabbitmqctl stop')

    # add a keyboard key event to stop the RabbitMQ server when the app is closed
    def on_keyboard(self, window, key, scancode, codepoint, modifier):
        """
        The method to stop the RabbitMQ server when the app is closed
        The key event is triggered when the escape key is pressed
        """
        log.info(
            f'window: {window}\n'
            f'key: {key}\n'
            f'scancode: {scancode}\n'
            f'codepoint: {codepoint}\n'
            f'modifier: {modifier}\n'
        )


def pre_init():
    """
    This function is called before the server is initialized
    """
    client = docker.from_env()

    # print all containers
    containers = client.containers.list()

    for container in containers:
        print(
            container.name,
            container.id,
            container.status
        )

    # if there is no docker container running named aij-messaging-server then run the server script 'aijinit'
    if not any(container.name == 'aij-messaging-server' for container in containers):
        os.system('aijinit')


def main():
    """
    The main function to run the server and publish the news articles to the RabbitMQ queue
    """
    print('Server is being initialized...')
    
    # initialize the server if it is not already initialized
    pre_init()

    # create an instance of the AIJKivy class
    app = AIJKivy()
    app.run()

    print('Server is now running...')


if __name__ == '__main__':
    main()



import os, sys, datetime, time
import urllib.request

from dotenv import load_dotenv
import docker

from psutil import process_iter
from signal import SIGTERM # or SIGKILL

load_dotenv()  # take environment variables from .env.

user_profile = os.environ['USERPROFILE']
SEP = os.path.sep

def main():
    """
    The main function to run the server and publish the news articles to the RabbitMQ queue
    """
    print('Server is being initialized...')
    
    # check first if docker is installed
    # if it has docker then run the following code
    if os.system('docker --version') == 0:
        
        print('Docker is installed!')
        
        client = docker.from_env()

        # print all containers
        containers = client.containers.list()

        for container in containers:
            print(
                container.name,
                container.id,
                container.status
            )

        # TODO: add this feature by accepting a command line argument such as --force
        # search any application running on port 5672 and close the process by its ID using sys package
        
        if len(sys.argv) > 0 and sys.argv[1] == '--force':
        
            if len(containers) > 0:
                # search if any container is running on port 5672 and close the process
                for container in containers:
                    if container.name == 'aij-messaging-server':
                        container.kill()
                        
                    if 5672 in container.ports:
                        container.kill()
                        
            for proc in process_iter():
                for conns in proc.connections(kind='inet'):
                    if conns.laddr.port == 5672:
                        proc.send_signal(SIGTERM) # or SIGKILL

        client.containers.run(
            image="rabbitmq:3-management", 
            name='aij-messaging-server', 
            ports={
                '5672': 5672,
                '15672': 15672
            },
            remove = True,
            detach=True, 
        )
        
        print('Server is now running using Docker...')
        
    # if docker is not installed then run the following code
    else:
        print('Docker is not installed!')
        os.system('rabbitmq-server -detached')
        print('Server is now running using local RabbitMQ server...')

    print('Server is now running...')


if __name__ == '__main__':
    main()


import os
from pathlib import Path
import openai

from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

user_profile = os.environ['USERPROFILE']
SEP = os.path.sep


def main():
    """This method extract the audio from a video
    The transcriptions API takes as input the audio file you want to transcribe and the desired output file format 
    for the transcription of the audio. They currently support multiple input and output file formats.
    """

    # find all the videos in the data directory using the glob module
    videos = [str(video) for video in Path('data').glob('*.mp4')]

    # transcribe each video
    for video in videos:
        #  ffmpeg data conversion from mp4 to wav
        audio = video.replace('.mp4', '.wav')
        if not os.path.exists(audio):
            os.system(
                f"ffmpeg -i {video} -vn -acodec pcm_s16le -ar 16000 -ac 1 {video.replace('.mp4', '.wav')}")

        # Transcriptions
        # Note: you need to be using OpenAI Python v0.27.0 for the code below to work
        # The transcriptions API takes as input the audio file you want to transcribe and the desired output file format for
        # the transcription of the audio. We currently support multiple input and output file formats.
        audio_file = open(video.replace('.mp4', '.wav'), "rb")
        transcript = openai.Audio.transcribe("whisper-1", audio_file)

        print(
            transcript
        )

        # Translations
        # The translations API takes as input the audio file in any of the supported languages and transcribes,
        # if necessary, the audio into english. This differs from our /Transcriptions endpoint since
        # the output is not in the original input language and is instead translated to english text.
        
        transcript = openai.Audio.translate("whisper-1", audio_file)

        print(
            transcript
        )



if __name__ == '__main__':
    main()



import json
import logging as log
import time
from threading import Timer, Thread
import logging as log
import os, sys
import time
import pika
from datetime import datetime

# using Kivy to create a GUI for the publisher and subscriber
from kivy.app import App
from kivy.core.window import Window
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.label import Label

import docker

from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

class VideoPublisher:
    """
    A class to publish news articles to a RabbitMQ queue using the NewsAPI
    @param api_key: the api key to access the NewsAPI
    @param host: the host name of the RabbitMQ server
    @param queue_name: the name of the queue to publish the news articles
    """

    def __init__(self, host='localhost', queue_name='video_stream'):
        self.host = host
        self.queue_name = queue_name
        # set up a connection to RabbitMQ
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host=self.host))
        self.channel = self.connection.channel()
        self.sources = [
            'https://www.youtube.com/watch?v=8Db3rluT76w&pp=ygULZnVubnkgbmV3cyA%3D',
        ]

    def publish(self):
        """
        Publish the news articles to the RabbitMQ queue one by one and save the news to the database
        """
        try:
            for _source in self.sources:
                
                _video_cmd = f'youtube-dl "{_source}"'
                
                os.system(_video_cmd)

                _body = json.dumps(
                    {
                        'id': _source.split("/")[-1],
                        'src': _source,
                        'at': time.time(),
                        'on': datetime.now(),
                        'by': 'AIJ'
                    }
                ).encode('utf-8')

                self.channel.basic_publish(exchange='', routing_key=self.queue_name, body=_body)

                log.info(
                    f"Published a news article to the queue: {_source}")

                # wait for 1 second before publishing the next article
                time.sleep(1)

        except Exception as api_exception:
            log.error(
                f"Could not request results from NewsAPI; {api_exception}")

        # do not close the connection until the message is delivered
        if self.connection.is_open:
            self.connection.close()

        # call the function again after 100 seconds because there are max. 100 results per page
        Timer(60 * 5, self.publish).start()

    def destroy(self):
        """
        Destroy the connection to the RabbitMQ server
        """
        self.connection.close()


class NewsSubscriber:
    """
    This class implements a RabbitMQ consumer.
    """

    def __init__(self, host='localhost', queue_name='video_stream', callback=None):
        """
        This method initializes the consumer.
        """
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host))
        self.channel = self.connection.channel()
        self.channel.queue_declare(queue=queue_name)
        self.callback = callback
        self.queue_name = queue_name

    def subscribe(self):
        """
        This method starts consuming messages from the RabbitMQ queue.
        """
        self.channel.basic_consume(
            queue=self.queue_name, on_message_callback=self.callback, auto_ack=True)
        self.channel.start_consuming()

    def destroy(self):
        """
        This method closes the connection to the RabbitMQ server.
        """
        if self.connection.is_open:
            self.connection.close()


class AIJKivy(App):
    """
    A class to create a GUI for the publisher and subscriber
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # create an instance of the VideoPublisher class
        self.publisher = VideoPublisher(
            host='localhost',
            queue_name='video_stream'
        )

        # create an instance of the NewsSubscriber class
        self.subscriber = NewsSubscriber(
            host='localhost',
            queue_name='video_stream',
            callback=self.print_news
        )

        self.title = 'AIJ News'
        self.icon = 'logo.png'

        # create a layout for the publisher
        self.publisher_layout = BoxLayout(orientation='vertical', padding=10)
        self.publisher_label = Label(text='Publisher', size_hint=(1, 0.90))
        self.publisher_button = Button(text='Publish', size_hint=(1, 0.10))

        # create a layout for the subscriber
        self.subscriber_layout = BoxLayout(orientation='vertical', padding=10)
        self.subscriber_label = Label(text='Subscriber', size_hint=(1, 0.90))
        self.subscriber_button = Button(text='Subscribe', size_hint=(1, 0.10))

        # create a layout for the main window
        self.main_layout = BoxLayout(orientation='horizontal', padding=10)

    def build(self):
        """
        Build the GUI for the publisher and subscriber
        """

        # add the label, text input and button to the publisher layout
        self.publisher_layout.add_widget(self.publisher_label)
        self.publisher_layout.add_widget(self.publisher_button)
        # bind the button to the publish method
        self.publisher_button.bind(on_press=self.publish)

        # add the label, text input and button to the subscriber layout
        self.subscriber_layout.add_widget(self.subscriber_label)
        self.subscriber_layout.add_widget(self.subscriber_button)
        # bind the button to the subscribe method
        self.subscriber_button.bind(on_press=self.subscribe)

        # add the publisher and subscriber layouts to the main layout
        self.main_layout.add_widget(self.publisher_layout)
        self.main_layout.add_widget(self.subscriber_layout)

        return self.main_layout

    def publish(self, *args):
        """
        The method to publish the news articles to the RabbitMQ queue
        """

        # starts a thread to publish the news articles to the RabbitMQ queue
        news_publisher_thread = Thread(target=self.publisher.publish)

        # start the thread
        news_publisher_thread.start()

    def subscribe(self, *args):
        """
        The method to subscribe to the RabbitMQ queue and receive the news articles
        """

        # starts a thread to subscribe to the RabbitMQ queue and receive the news articles
        news_subscriber_thread = Thread(target=self.subscriber.subscribe)

        # start the thread
        news_subscriber_thread.start()

    def print_news(self, ch, method, properties, body):
        """
        This method is called when a message is received from the RabbitMQ queue.
        """
        response = body.decode('utf-8')
        print(
            f'-------------------------\n'
            f'{response}'
            f'-------------------------\n'
        )
        time.sleep(1)

def pre_init():
    """
    This function is called before the server is initialized
    """
    client = docker.from_env()

    # print all containers
    containers = client.containers.list()

    for container in containers:
        print(
            container.name,
            container.id,
            container.status
        )

    # if there is no docker container running named aij-messaging-server then run the server script 'aijinit'
    if not any(container.name == 'aij-messaging-server' for container in containers):
        os.system('aijinit')

def main():
    """
    The main function to run the server and publish the news articles to the RabbitMQ queue
    """
    print('Server is being initialized...')

    # initialize the server if it is not already initialized
    pre_init()
    
    # create an instance of the AIJKivy class
    app = AIJKivy()
    app.run()

    print('Server is now running...')


if __name__ == '__main__':
    main()



import os
import openai

from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

openai.api_key = os.getenv("OPENAI_API_KEY")

response = openai.Completion.create(
    model="text-davinci-003",
    prompt="The following is a conversation with an AI journalist. The assistant is helpful, creative, clever, and very friendly.\n\nHuman: Hello, who are you?\nAI: I am an AI created by OpenAI. How can I help you today?\nHuman: I'd like to read the latest news.\nAI:",
    temperature=0.9,
    max_tokens=150,
    top_p=1,
    frequency_penalty=0.0,
    presence_penalty=0.6,
    stop=[" Human:", " AI:"]
)

print(
    response
)
